//
//  VertexExtractor.h
//  ConvexHull2d
//
//  Created by kumada on 2013/06/08.
//  Copyright (c) 2013年 kumada. All rights reserved.
//

#ifndef ConvexHull2d_VertexExtractor_h
#define ConvexHull2d_VertexExtractor_h

#include "Point.h"
#include "Concepts.h"
#include <vector>

template<typename T>
class VertexExtractor
{
        BOOST_CLASS_REQUIRE(T, geo, IntegerConcept);
        
public:
        typedef geo::Point<T, geo::TwoDimension> Point;
        typedef std::unique_ptr<Point>           PointPtr;
        
        VertexExtractor(
                const std::vector<PointPtr>& original_points,
                const std::vector<int>&      names,
                const std::vector<int>&      vertices
        )
                : original_points_(original_points)
                , sorted_to_original_indices_(names)
                , ordered_indices_()
        {
                int first = search_first_vertex(vertices);
                int i = first;
                while ( true ) {
                        int index = vertices[i];
                        ordered_indices_.push_back(index);
                        i = index;
                        if ( i == first ) {
                                break;
                        }
                }
        }

        const PointPtr& get_vertex(std::size_t i) const
        {
                return original_points_[sorted_to_original_indices_[ordered_indices_[i]]];
        }
        
        const std::size_t get_size() const
        {
                return ordered_indices_.size();
        }
        
private:
        int search_first_vertex(const std::vector<int>& vertices) const
        {
                for ( std::size_t i = 0; i < vertices.size(); ++i ) {
                        if ( vertices[i] != -1 ) {
                                return static_cast<int>(i);
                        }
                }
                return 0;
        }
        const std::vector<PointPtr>& original_points_;
        const std::vector<int>&      sorted_to_original_indices_;
        std::vector<int>             ordered_indices_;
};


#endif
