//
//  main.cpp
//  ConvexHull2d
//
//  Created by kumada on 2013/06/02.
//  Copyright (c) 2013年 kumada. All rights reserved.
//
#if(UNIT_TEST)
#define BOOST_TEST_MAIN
#define BOOST_TEST_DYN_LINK

#include <boost/test/unit_test.hpp>
#include <iostream>

BOOST_AUTO_TEST_CASE(TEST_main)
{
        std::cout << "main\n";
}

#else // UNIT_TEST

#include "ConvexHull2d.h"
#include <random>
#include <opencv2/opencv.hpp>
#include <boost/range/algorithm/for_each.hpp>

using namespace geo;
int main(int argc, const char * argv[])
{
        constexpr int width  {300};
        constexpr int height {300};
        constexpr int offset {10};

        typedef ConvexHull2d<int>::Point    Point;
        typedef ConvexHull2d<int>::PointPtr PointPtr;

        //_/_/_/ generate random points which exist inside a circle _/_/_/
        
        constexpr int n {1000};
        std::vector<PointPtr> points;
        points.reserve(n);

        std::random_device seed {};
        int s = seed();
        std::cout << "current seed : " << s << std::endl;
        std::default_random_engine e {s};
        std::uniform_int_distribution<int> d {offset, width - offset};

        const Point center {{width / 2, height / 2}};
        int i = 0;
        const double r = width / 2 - offset;
        while ( i < n ) {
                PointPtr p = PointPtr{new Point{{d(e), d(e)}} };
                //double d = norm<int, TwoDimension>(*p - center);
                //if ( d <= r ) {
                        points.push_back(std::move(p));
                        ++i;
                //}
        }
        std::cout << "the number of random points : " << points.size() << std::endl;
        
        //_/_/_/ execute the convex hull _/_/_/
        
        ConvexHull2d<int> ch2d {points};
        ch2d.execute();

        //_/_/_/ display the result _/_/_/
        
        // create an image
        cv::Mat image {height, width, CV_8UC1, 255};
        
        // draw input points
        boost::for_each(points, [&](const PointPtr& p){
                cv::circle(image, {p->at(0), p->at(1)}, 1, 0, CV_FILLED);
        });
        
        // draw points which constitute the convex hull
        auto ccextractor = ch2d.extract_ccvertices();
        std::size_t size = ccextractor.get_size();
        std::cout << "the number of points which constitute convex hull : " << size << std::endl;
        for ( std::size_t i = 0; i < size; ++i ) {
                const Point& p0 = *ccextractor.get_vertex(i);
                std::size_t t = (i + 1) % size;
                const Point& p1 = *ccextractor.get_vertex(t);
                cv::line(image, {p0[0], p0[1]}, {p1[0], p1[1]}, 128);
        }
        
        cv::imwrite("/Users/kumada/Data/computational-geometry/result.png", image);
        cv::imshow("result", image);
        cv::waitKey();
        
        return 0;
}
#endif // UNIT_TEST