//
//  Concepts.h
//  LevelSetMethod2
//
//  Created by kumada on 2012/11/18.
//  Copyright (c) 2012年 kumada. All rights reserved.
//

#ifndef SymbolicPerturbation2d_Concepts_h
#define SymbolicPerturbation2d_Concepts_h

#include <type_traits>
#include <boost/multiprecision/cpp_int.hpp>

namespace geo
{
        /** Concept for the dimension
                @tparam T the dimension type
                @see DimensionTypes.h
        */
        template<typename T>
        struct DimensionConcept
        {
                void constraints()
                {
                        (void)T::Dimension_;
                        static_assert(T::Dimension_ == 2 || T::Dimension_ == 3, "Invalid Dimension Type");
                }
        }; // struct DimensionConcept

        /** Concept for the integer
                @tparam T the integer type
        */
        template<typename T>
        struct IntegerConcept
        {
                void constraints()
                {
                        static_assert(
                                std::is_same<T, int>::value ||
                                std::is_same<T, boost::multiprecision::cpp_int>::value ||
                                std::is_same<T, boost::multiprecision::int128_t>::value ||
                                std::is_same<T, boost::multiprecision::int256_t>::value ||
                                std::is_same<T, boost::multiprecision::int512_t>::value ||
                                std::is_same<T, boost::multiprecision::int1024_t>::value,
                                "Invalid Integer Type"
                        );
                }
        }; // struct IntegerConcept

} // namespace geo
#endif // SymbolicPerturbation2d_Concepts_h
