//
//  SymbolicPerturbation2d.cpp
//  SymbolicPerturbation2d
//
//  Created by kumada on 2013/05/31.
//  Copyright (c) 2013年 kumada. All rights reserved.
//
#if(UNIT_TEST)
#define BOOST_TEST_DYN_LINK
#include <boost/test/unit_test.hpp>
#include "SymbolicPerturbation2d.h"
#include <boost/multiprecision/cpp_int.hpp>
#include "Point.h"
#include "DimensionTypes.h"
#include <random>
#include <boost/range/algorithm/for_each.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/range/algorithm/sort.hpp>
#include <boost/range/algorithm/transform.hpp>

namespace mp = boost::multiprecision;
using namespace geo;

class SymbolicPerturbation2dTester
{
public:
        static void test_sort0()
        {
                typedef SymbolicPerturbation2d<int> Sp;
                
                std::vector<Sp::PointPtr> original_points;
                std::vector<int> names;
                Sp sp {original_points, names};

                const std::array<int, 3> ilist = {7, 5, 9};
                std::array<int, 3> jlist {};
                bool is_even_permutation = sp.sort(ilist, jlist);
                int answers[] = {9, 7, 5};
                int k = 0;
                boost::for_each(jlist, [&](int x){ BOOST_CHECK_EQUAL(x, answers[k++]); });
                BOOST_CHECK(is_even_permutation);
        }
        
        static void test_sort1()
        {
                typedef SymbolicPerturbation2d<int> Sp;
                
                std::vector<Sp::PointPtr> original_points;
                std::vector<int> names;
                Sp sp {original_points, names};

                const std::array<int, 3> ilist = {7, 9, 5};
                std::array<int, 3> jlist {};
                bool is_even_permutation = sp.sort(ilist, jlist);
                int answers[] = {9, 7, 5};
                int k = 0;
                boost::for_each(jlist, [&](int x){ BOOST_CHECK_EQUAL(x, answers[k++]); });
                BOOST_CHECK(!is_even_permutation);
        }

        static void test_sort2()
        {
                typedef SymbolicPerturbation2d<int> Sp;
                
                std::vector<Sp::PointPtr> original_points;
                std::vector<int> names;
                Sp sp {original_points, names};

                const std::array<int, 3> ilist = {9, 7, 5};
                std::array<int, 3> jlist {};
                bool is_even_permutation = sp.sort(ilist, jlist);
                int answers[] = {9, 7, 5};
                int k = 0;
                boost::for_each(jlist, [&](int x){ BOOST_CHECK_EQUAL(x, answers[k++]); });
                BOOST_CHECK(is_even_permutation);
        }
        
        static void test_sort3()
        {
                typedef SymbolicPerturbation2d<int> Sp;
                
                std::vector<Sp::PointPtr> original_points;
                std::vector<int> names;
                Sp sp {original_points, names};

                const std::array<int, 3> ilist = {3, 0, 1};
                std::array<int, 3> jlist {};
                bool is_even_permutation = sp.sort(ilist, jlist);
                int answers[] = {3, 1, 0};
                int k = 0;
                boost::for_each(jlist, [&](int x){ BOOST_CHECK_EQUAL(x, answers[k++]); });
                BOOST_CHECK(!is_even_permutation);
        }

        
        static void test_judge()
        {
                typedef SymbolicPerturbation2d<int> Sp;
                
                std::vector<Sp::PointPtr> original_points;
                std::vector<int> names;
                Sp sp {original_points, names};

                int sign = 1;
                bool is_even_permutation = true;
                BOOST_CHECK(sp.judge(sign, is_even_permutation));
                
                sign = -1;
                BOOST_CHECK(!sp.judge(sign, is_even_permutation));
        }
        
        template<typename T>
        static void test_compute_sign_of_determinant0()
        {
                typedef SymbolicPerturbation2d<T> Sp;
                typename Sp::PointPtr p0, p1, p2;
                std::vector<typename Sp::PointPtr> original_points;
                std::vector<int> names;
                Sp sp {original_points, names};
                
                T a0 = 2;
                T a1 = 3;
                BOOST_CHECK(sp.compute_sign_of_determinant(a0, a1) == 1);
        }
        
        template<typename T>
        static void test_compute_sign_of_determinant1()
        {
                typedef SymbolicPerturbation2d<T> Sp;
                
                std::vector<typename Sp::PointPtr> original_points;
                std::vector<int> names;
                Sp sp {original_points, names};
                
                T a0 = 2;
                T a1 = 2;
                BOOST_CHECK(sp.compute_sign_of_determinant(a0, a1) == 0);
        }
        
        template<typename T>
        static void test_compute_sign_of_determinant_with_array0()
        {
                typedef SymbolicPerturbation2d<T> Sp;
                
                std::vector<typename Sp::PointPtr> original_points;
                std::vector<int> names;
                Sp sp {original_points, names};
                
                const std::array<T, 3> ia1 = {1, 2, 3};
                const std::array<T, 3> ia2 = {4, 5, 6};
                
                BOOST_CHECK(sp.compute_sign_of_determinant(ia1, ia2) == 0);
        }

        template<typename T>
        static void test_compute_sign_of_determinant_with_array1()
        {
                typedef SymbolicPerturbation2d<T> Sp;
                typename Sp::PointPtr p0, p1, p2;
                std::vector<typename Sp::PointPtr> original_points;
                std::vector<int> names;
                Sp sp {original_points, names};
                
                const std::array<T, 3> ia1 = {1, 2, 3};
                const std::array<T, 3> ia2 = {4, 5, 3};
                
                BOOST_CHECK(sp.compute_sign_of_determinant(ia1, ia2) == -1);
        }
      
        template<typename T>
        static void test_turns_left0()
        {
                typedef SymbolicPerturbation2d<T> Sp;
                
                std::vector<typename Sp::PointPtr> original_points(3);
                // y = 2 - x
                original_points[0] = typename Sp::PointPtr(new typename Sp::Point{{0, 0}});
                original_points[1] = typename Sp::PointPtr(new typename Sp::Point{{1, 1}});
                original_points[2] = typename Sp::PointPtr(new typename Sp::Point{{2, 2}});
                
                std::vector<int> names = {0, 1, 2};
                Sp sp{original_points, names};
                BOOST_CHECK(!sp.turns_left(0, 1, 2));
        }
        
        template<typename T>
        static void test_turns_left1()
        {
                typedef SymbolicPerturbation2d<T> Sp;
                
                std::vector<typename Sp::PointPtr> original_points(3);
                // y = x
                original_points[0] = typename Sp::PointPtr(new typename Sp::Point{{0, 0}});
                original_points[1] = typename Sp::PointPtr(new typename Sp::Point{{1, 1}});
                original_points[2] = typename Sp::PointPtr(new typename Sp::Point{{1, 1}});

                std::vector<int> names = {0, 1, 2};
                Sp sp{original_points, names};
                BOOST_CHECK(sp.turns_left(0, 1, 2));
        }
        
        template<typename T>
        static void test_turns_left2()
        {
                typedef SymbolicPerturbation2d<T> Sp;
                
                std::vector<typename Sp::PointPtr> original_points(3);
                
                original_points[0] = typename Sp::PointPtr(new typename Sp::Point{{0, 0}});
                original_points[1] = typename Sp::PointPtr(new typename Sp::Point{{1, 0}});
                original_points[2] = typename Sp::PointPtr(new typename Sp::Point{{2, 0}});
                
                
                std::vector<int> names = {0, 1, 2};
                Sp sp {original_points, names};
                BOOST_CHECK(sp.turns_left(0, 1, 2));
        }
        
        template<typename T>
        static void test_turns_left3()
        {
                typedef SymbolicPerturbation2d<T> Sp;
                
                std::vector<typename Sp::PointPtr> original_points(3);
                
                original_points[0] = typename Sp::PointPtr(new typename Sp::Point{{0, 0}});
                original_points[1] = typename Sp::PointPtr(new typename Sp::Point{{1, 0}});
                original_points[2] = typename Sp::PointPtr(new typename Sp::Point{{1, 0}});

                std::vector<int> names = {0, 1, 2};
                Sp sp {original_points, names};
                BOOST_CHECK(!sp.turns_left(0, 1, 2));
        }
        
        template<typename T>
        static void test_turns_left4()
        {
                typedef SymbolicPerturbation2d<T> Sp;
                
                std::vector<typename Sp::PointPtr> original_points(3);
                
                original_points[0] = typename Sp::PointPtr(new typename Sp::Point{{0, 0}});
                original_points[1] = typename Sp::PointPtr(new typename Sp::Point{{0, 0}});
                original_points[2] = typename Sp::PointPtr(new typename Sp::Point{{0, 0}});
                
                std::vector<int> names = {0, 1, 2};
                Sp sp {original_points, names};
                BOOST_CHECK(!sp.turns_left(0, 1, 2));
                
                
        }
        
        
};

BOOST_AUTO_TEST_CASE(TEST_SymbolicPerturbation2d)
{
        std::cout << "SymbolicPerturbation2d\n";
        
        SymbolicPerturbation2dTester::test_sort0();
        SymbolicPerturbation2dTester::test_sort1();
        SymbolicPerturbation2dTester::test_sort2();
        SymbolicPerturbation2dTester::test_sort3();

        SymbolicPerturbation2dTester::test_judge();
        SymbolicPerturbation2dTester::test_compute_sign_of_determinant0<int>();
        SymbolicPerturbation2dTester::test_compute_sign_of_determinant1<int>();
        SymbolicPerturbation2dTester::test_compute_sign_of_determinant0<mp::cpp_int>();
        SymbolicPerturbation2dTester::test_compute_sign_of_determinant1<mp::cpp_int>();

        SymbolicPerturbation2dTester::test_compute_sign_of_determinant_with_array0<int>();
        SymbolicPerturbation2dTester::test_compute_sign_of_determinant_with_array0<mp::cpp_int>();
        SymbolicPerturbation2dTester::test_compute_sign_of_determinant_with_array1<int>();
        SymbolicPerturbation2dTester::test_compute_sign_of_determinant_with_array1<mp::cpp_int>();

        SymbolicPerturbation2dTester::test_turns_left0<int>();
        SymbolicPerturbation2dTester::test_turns_left0<mp::cpp_int>();

        SymbolicPerturbation2dTester::test_turns_left1<int>();
        SymbolicPerturbation2dTester::test_turns_left1<mp::cpp_int>();

        SymbolicPerturbation2dTester::test_turns_left2<int>();
        SymbolicPerturbation2dTester::test_turns_left2<mp::cpp_int>();

        SymbolicPerturbation2dTester::test_turns_left3<int>();
        SymbolicPerturbation2dTester::test_turns_left3<mp::cpp_int>();

        SymbolicPerturbation2dTester::test_turns_left4<int>();
        SymbolicPerturbation2dTester::test_turns_left4<mp::cpp_int>();
}
#endif // UNIT_TEST